#!/usr/bin/python3
"""
Allow buildbot to run jobs dynamically defined a a project repo
Requires docker
"""

# TODO: change "ghcontext" in master.cfg to interpolate the job name
# TODO: write upload script
# TODO: write hook (perhaps the default hook is ok), authentication for hook
# TODO: write hook job, maybe also a dockerfile?
# TODO: what to do about non-docker jobs

from os import listdir, walk, getenv
from os.path import exists, isfile, join
import requests
import re
from buildbot.plugins import steps, util

from .lib.renderers import *


def find_artifacts(directory="out"):
    """
    find artifacts (any file) in a directory
    """
    for _ in listdir(directory):
        if not isfile(join(directory, _)):
            continue
        return join(directory, _)


def add_parent_step(build_factory):
    """
    Add a step to the parent build factory that will trigger the child hooks
    """

    build_factory.addStep(steps.ShellCommand(
        name="download worker",
        command=["mkdir", "-p", join(getenv['HOME'], '.local/bin'), "&&", "wget", "-O", "https://git.bitmessage.org/Bitmessage/buildbot_multibuild/raw/branch/master/lib/worker_multibuild.py", join(getenv['HOME'], '.local/bin/worker_multibuild.py')]
    ))

    build_factory.addStep(
        steps.ShellCommand(
            name="Execute worker script",
            command=[
                "python",
                join(getenv['HOME'], '.local/bin/worker_multibuild.py'),
                util.Interpolate("%(prop:jobname)s"),
                util.Property('repository'),
                util.Property('branch'),
                "https://buildbot.bitmessage.org",
                util.Interpolate("%(prop:os_codename)s"),
            ],
        )
    )


def add_child_sh_steps(build_factory, directory=".buildbot"):
    """
    Add a step to the download, build and test factory
    """

    build_factory.addStep(
        steps.ShellCommand(
            name=util.Interpolate("build_%(prop:jobname)s"),
            command=util.Interpolate("%(kw:directory)s/%(prop:jobname)s/build.sh", directory=directory),
            doStepIf=is_build_script_available,
            hideStepIf=isnt_build_script_available,
        )
    )

    build_factory.addStep(
        steps.ShellCommand(
            name= util.Interpolate("test_%(prop:jobname)s"),
            command=util.Interpolate("%(kw:directory)s/%(prop:jobname)s/test.sh", directory=directory),
            doStepIf=is_test_script_available,
            hideStepIf=isnt_test_script_available,
        )
    )


if __name__ == "__main__":
    # expect jobname, repository, branch, buildbotUrl from command line
    import sys

    if len(sys.argv) == 6:
        jobname = sys.argv[1]
        repository = sys.argv[2]
        branch = sys.argv[3]
        buildbotUrl = sys.argv[4]

        trigger_child_hooks(buildbotUrl, repository, branch, jobname, ".buildbot", is_build_script_available, is_test_script_available)
    else:
        print(
            "Usage: python3 multibuild.py <jobname> <repository> <branch> <buildbotUrl> "
        )
